@@ -21,11 +21,12 @@
 
 #include "terminal-intl.h"
 #include "terminal-accels.h"
+#include "terminal-widget.h"
 #include "terminal-window.h"
 #include "terminal.h"
 #include <string.h>
 #include <stdlib.h>
-#include <libzvt/libzvt.h>
+#include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <gdk/gdkx.h>
@@ -136,6 +137,8 @@ static void hide_menubar_callback         (GtkWidget      *menuitem,
                                            TerminalWindow *window);
 static void fullscreen_callback           (GtkWidget      *menuitem,
                                            TerminalWindow *window);
+static void set_title_callback            (GtkWidget      *menuitem,
+                                           TerminalWindow *window);
 static void reset_callback                (GtkWidget      *menuitem,
                                            TerminalWindow *window);
 static void reset_and_clear_callback      (GtkWidget      *menuitem,
@@ -156,7 +159,6 @@ static void set_menuitem_text (GtkWidget  *mi,
                                gboolean    strip_mnemonic);
 
 static gpointer parent_class;
-static guint signals[LAST_SIGNAL] = { 0 };
 
 GType
 terminal_window_get_type (void)
@@ -366,28 +368,28 @@ terminal_window_init (TerminalWindow *window)
                             accel_group);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
 
-  append_menuitem (menu, _("_New window"), ACCEL_PATH_NEW_WINDOW,
+  append_menuitem (menu, _("_New Window"), ACCEL_PATH_NEW_WINDOW,
                    G_CALLBACK (new_window_callback),
                    window);
 
-  mi = append_menuitem (menu, _("New _tab"), ACCEL_PATH_NEW_TAB,
+  mi = append_menuitem (menu, _("New _Tab"), ACCEL_PATH_NEW_TAB,
                         G_CALLBACK (new_tab_callback),
                         window);
 
   /* This is fairly bogus to have here but I don't know
    * where else to put it really
    */
-  append_menuitem (menu, _("New _profile..."), NULL,
-                   G_CALLBACK (new_configuration_callback), window);
+  append_menuitem (menu, _("New _Profile..."), ACCEL_PATH_NEW_PROFILE,
+		   G_CALLBACK (new_configuration_callback), window);
   
   mi = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
-  append_menuitem (menu, _("_Close window"), ACCEL_PATH_CLOSE_WINDOW,
+  append_menuitem (menu, _("_Close Window"), ACCEL_PATH_CLOSE_WINDOW,
                    G_CALLBACK (close_window_callback),
                    window);
   
-  append_menuitem (menu, _("C_lose tab"), ACCEL_PATH_CLOSE_TAB,
+  append_menuitem (menu, _("C_lose Tab"), ACCEL_PATH_CLOSE_TAB,
                    G_CALLBACK (close_tab_callback),
                    window);
   
@@ -416,7 +418,7 @@ terminal_window_init (TerminalWindow *window)
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   
   window->priv->edit_config_menuitem =
-    append_menuitem (menu, _("C_urrent profile..."), NULL,
+    append_menuitem (menu, _("C_urrent Profile..."), NULL,
                      G_CALLBACK (edit_configuration_callback), window);
 
   append_menuitem (menu, _("_Keybindings..."), NULL,
@@ -435,10 +437,10 @@ terminal_window_init (TerminalWindow *window)
                             accel_group);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
   
-  append_menuitem (menu, _("Hide menu_bar"), NULL,
+  append_menuitem (menu, _("Hide Menu_bar"), ACCEL_PATH_TOGGLE_MENUBAR,
                    G_CALLBACK (hide_menubar_callback), window);
 
-  mi = append_menuitem (menu, _("_Full screen"), NULL,
+  mi = append_menuitem (menu, _("_Full Screen"), ACCEL_PATH_FULL_SCREEN,
                         G_CALLBACK (fullscreen_callback), window);
   window->priv->fullscreen_menuitem = mi;
   
@@ -462,10 +464,13 @@ terminal_window_init (TerminalWindow *window)
     append_menuitem (menu, _("_Profile"), NULL,
                      NULL, NULL);
   
-  append_menuitem (menu, _("_Reset"), NULL,
+  append_menuitem (menu, _("_Set Title..."), ACCEL_PATH_SET_TERMINAL_TITLE,
+                   G_CALLBACK (set_title_callback), window);
+  
+  append_menuitem (menu, _("_Reset"), ACCEL_PATH_RESET,
                    G_CALLBACK (reset_callback), window);
 
-  append_menuitem (menu, _("Reset and C_lear"), NULL,
+  append_menuitem (menu, _("Reset and C_lear"), ACCEL_PATH_RESET_AND_CLEAR,
                    G_CALLBACK (reset_and_clear_callback), window);
 
   mi = append_menuitem (window->priv->menubar,
@@ -479,11 +484,11 @@ terminal_window_init (TerminalWindow *window)
                             accel_group);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
 
-  mi = append_menuitem (menu, _("_Previous tab"), NULL,
+  mi = append_menuitem (menu, _("_Previous Tab"), ACCEL_PATH_PREV_TAB,
                         G_CALLBACK (previous_tab_callback), window);
   window->priv->previous_tab_menuitem = mi;
   
-  mi = append_menuitem (menu, _("_Next tab"), NULL,
+  mi = append_menuitem (menu, _("_Next Tab"), ACCEL_PATH_NEXT_TAB,
                         G_CALLBACK (next_tab_callback), window);
   window->priv->next_tab_menuitem = mi;
 
@@ -500,8 +505,9 @@ terminal_window_init (TerminalWindow *window)
                             accel_group);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
 
-  append_menuitem (menu, _("_Help Contents"), NULL,
-                   G_CALLBACK (help_callback), window);
+  mi = append_stock_menuitem (menu, GTK_STOCK_HELP, NULL,
+			      G_CALLBACK (help_callback), window);
+  set_menuitem_text (mi, _("_Help Contents"), FALSE);
   
   mi = append_stock_menuitem (menu, GNOME_STOCK_ABOUT, NULL,
                               G_CALLBACK (about_callback), window);
@@ -686,6 +692,29 @@ update_copy_sensitivity (TerminalWindow *window)
   gtk_widget_set_sensitive (window->priv->copy_menuitem, can_copy);
 }
 
+static void
+update_tab_sensitivity (TerminalWindow *window)
+{
+  int page_num;
+  GtkWidget *notebook;
+  gboolean on_last_page;
+
+  notebook = window->priv->notebook;
+  page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+  
+  gtk_widget_set_sensitive (window->priv->previous_tab_menuitem,
+                            page_num > 0);
+
+  /* FIXME
+   * http://bugzilla.gnome.org/show_bug.cgi?id=73229
+   */
+  on_last_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
+                                            page_num + 1) == NULL;
+  
+  gtk_widget_set_sensitive (window->priv->next_tab_menuitem,
+                            !on_last_page);
+}
+
 static void
 selection_changed_callback (TerminalScreen *screen,
                             TerminalWindow *window)
@@ -698,10 +727,10 @@ terminal_window_add_screen (TerminalWindow *window,
                             TerminalScreen *screen)
 {
   TerminalWindow *old;
-  ZvtTerm *term;
+  GtkWidget *term;
   GtkWidget *hbox;
   GtkWidget *scrollbar;
-  GtkWidget *label;
+  GtkWidget *label;  
   
   old = terminal_screen_get_window (screen);
 
@@ -746,13 +775,13 @@ terminal_window_add_screen (TerminalWindow *window,
                     G_CALLBACK (selection_changed_callback),
                     window);
   
-  term = ZVT_TERM (terminal_screen_get_widget (screen));
+  term = terminal_screen_get_widget (screen);
 
   gtk_box_pack_start (GTK_BOX (hbox),
                       GTK_WIDGET (term), TRUE, TRUE, 0);
   
   gtk_range_set_adjustment (GTK_RANGE (scrollbar),
-                            term->adjustment);
+                            terminal_widget_get_scroll_adjustment (term));
 
   gtk_widget_show_all (hbox);
   terminal_window_update_scrollbar (window, screen);
@@ -764,6 +793,7 @@ terminal_window_add_screen (TerminalWindow *window,
                             label);
 
   reset_tab_menuitems (window);
+  update_tab_sensitivity (window);
   
   /* Busting out the GtkNotebook crackrock features */
   gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (window->priv->notebook),
@@ -774,6 +804,27 @@ terminal_window_add_screen (TerminalWindow *window,
    * the size request right.
    */
   gtk_widget_realize (GTK_WIDGET (term));
+
+  {
+    /* Match size to current active screen */
+    int current_width, current_height;
+    GtkWidget *current_widget;
+
+    if (window->priv->active_term)
+      current_widget = terminal_screen_get_widget (window->priv->active_term);
+    else
+      current_widget = NULL;
+    
+    if (current_widget)
+      terminal_widget_get_size (current_widget, &current_width, &current_height);
+    else
+      {
+        current_width = 80;
+        current_height = 24;
+      }
+
+    terminal_widget_set_size (term, current_width, current_height);
+  }
   
   /* Make the first-added screen the active one */
   if (window->priv->terms == NULL)
@@ -816,13 +867,15 @@ terminal_window_remove_screen (TerminalWindow *window,
   g_object_unref (G_OBJECT (screen));
   
   update_notebook (window);
-  reset_tab_menuitems (window);
 
   /* Put ourselves back in a sane state */
   if (window->priv->active_term == NULL &&
       window->priv->terms)
     terminal_window_set_active (window, window->priv->terms->data);
 
+  reset_tab_menuitems (window);
+  update_tab_sensitivity (window);
+  
   /* Close window if no more terminals */
   if (window->priv->terms == NULL)
     gtk_widget_destroy (GTK_WIDGET (window));
@@ -860,6 +913,11 @@ terminal_window_set_menubar_visible (TerminalWindow *window,
       gtk_container_remove (GTK_CONTAINER (window->priv->main_vbox),
                             window->priv->menubar);
     }
+
+  if (window->priv->active_term)
+    {
+      terminal_window_set_size (window, window->priv->active_term, TRUE);
+    }
 }
 
 gboolean
@@ -868,23 +926,25 @@ terminal_window_get_menubar_visible (TerminalWindow *window)
   return window->priv->menubar_visible;
 }
 
-
-#define PADDING 0 /* from zvtterm.c */
 void
 terminal_window_set_size (TerminalWindow *window,
                           TerminalScreen *screen,
                           gboolean        even_if_mapped)
 {
   /* Owen's hack from gnome-terminal */
-  ZvtTerm *term;
   GtkWidget *widget;
   GtkWidget *app;
   GtkRequisition toplevel_request;
   GtkRequisition widget_request;
-  gint w, h;
+  int w, h;
+  int char_width;
+  int char_height;
+  int grid_width;
+  int grid_height;
+  int xpad;
+  int ypad;
   
   widget = terminal_screen_get_widget (screen);
-  term = ZVT_TERM (widget);
   
   app = gtk_widget_get_toplevel (widget);
   g_assert (app != NULL);
@@ -902,12 +962,13 @@ terminal_window_set_size (TerminalWindow *window,
   
   w = toplevel_request.width - widget_request.width;
   h = toplevel_request.height - widget_request.height;
+
+  terminal_widget_get_cell_size (widget, &char_width, &char_height);
+  terminal_widget_get_size (widget, &grid_width, &grid_height);
+  terminal_widget_get_padding (widget, &xpad, &ypad);
   
-  w += widget->style->xthickness * 2 + PADDING;
-  h += widget->style->ythickness * 2;
-  
-  w += term->charwidth * term->grid_width;
-  h += term->charheight * term->grid_height;
+  w += xpad + char_width * grid_width;
+  h += ypad + char_height * grid_height;
   
   if (even_if_mapped)
     gtk_window_resize (GTK_WINDOW (app), w, h);
@@ -919,16 +980,14 @@ void
 terminal_window_set_active (TerminalWindow *window,
                             TerminalScreen *screen)
 {
-  ZvtTerm *term;
   GtkWidget *widget;
   TerminalProfile *profile;
   
   if (window->priv->active_term == screen)
     return;
-
+  
   widget = terminal_screen_get_widget (screen);
-  term = ZVT_TERM (widget);
-
+  
   profile = terminal_screen_get_profile (screen);
   
   if (!GTK_WIDGET_REALIZED (widget))
@@ -957,9 +1016,9 @@ terminal_window_set_active (TerminalWindow *window,
                                  gtk_notebook_page_num (GTK_NOTEBOOK (window->priv->notebook),
                                                         screen_get_hbox (screen)));
 
-  /* set initial size of window */
-  terminal_window_set_size (window, screen, FALSE);
-
+  /* set size of window to current grid size */
+  terminal_window_set_size (window, screen, TRUE);
+  
   gtk_widget_grab_focus (terminal_screen_get_widget (window->priv->active_term));
 }
 
@@ -1056,7 +1115,6 @@ notebook_page_switched_callback (GtkWidget       *notebook,
 {
   GtkWidget* page_widget;
   TerminalScreen *screen;
-  gboolean on_last_page;
   GtkWidget *menu_item;
   
   page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
@@ -1070,18 +1128,8 @@ notebook_page_switched_callback (GtkWidget       *notebook,
 
   terminal_window_set_active (window, screen);
 
-  gtk_widget_set_sensitive (window->priv->previous_tab_menuitem,
-                            page_num > 0);
-
-  /* FIXME
-   * http://bugzilla.gnome.org/show_bug.cgi?id=73229
-   */
-  on_last_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                                            page_num + 1) == NULL;
+  update_tab_sensitivity (window);
   
-  gtk_widget_set_sensitive (window->priv->next_tab_menuitem,
-                            !on_last_page);
-
   menu_item = screen_get_menuitem (screen);
   if (menu_item &&
       screen == window->priv->active_term)
@@ -1177,44 +1225,47 @@ void
 terminal_window_update_geometry (TerminalWindow *window)
 {
   GdkGeometry hints;
-  ZvtTerm *term;
   GtkWidget *widget;  
-
+  int char_width;
+  int char_height;
+  
   if (window->priv->active_term == NULL)
     return;
   
   widget = terminal_screen_get_widget (window->priv->active_term);
-  term = ZVT_TERM (widget);
   
   /* We set geometry hints from the active term; best thing
    * I can think of to do. Other option would be to try to
    * get some kind of union of all hints from all terms in the
    * window, but that doesn't make too much sense.
    */
-
-  if (term->charwidth != window->priv->old_char_width ||
-      term->charheight != window->priv->old_char_height)
+  terminal_widget_get_cell_size (widget, &char_width, &char_height);
+  
+  if (char_width != window->priv->old_char_width ||
+      char_height != window->priv->old_char_height)
     {
-      /* padding copied from zvt size request. */
+      int xpad, ypad;
       
-      /* FIXME Since we're using xthickness/ythickness we need to change
-       * the hints when the theme changes.
-       */  
-      hints.base_width = (GTK_WIDGET (term)->style->xthickness * 2) + PADDING;
-      hints.base_height =  (GTK_WIDGET (term)->style->ythickness * 2);
+      /* FIXME Since we're using xthickness/ythickness to compute
+       * padding we need to change the hints when the theme changes.
+       */
+      terminal_widget_get_padding (widget, &xpad, &ypad);
+      
+      hints.base_width = xpad;
+      hints.base_height = ypad;
 
 #define MIN_WIDTH_CHARS 4
 #define MIN_HEIGHT_CHARS 2
       
-      hints.width_inc = term->charwidth;
-      hints.height_inc = term->charheight;
+      hints.width_inc = char_width;
+      hints.height_inc = char_height;
 
       /* min size is min size of just the geometry widget, remember. */
       hints.min_width = hints.base_width + hints.width_inc * MIN_WIDTH_CHARS;
       hints.min_height = hints.base_height + hints.height_inc * MIN_HEIGHT_CHARS;
       
       gtk_window_set_geometry_hints (GTK_WINDOW (window),
-                                     GTK_WIDGET (term),
+                                     widget,
                                      &hints,
                                      GDK_HINT_RESIZE_INC |
                                      GDK_HINT_MIN_SIZE |
@@ -1534,7 +1585,7 @@ copy_callback (GtkWidget      *menuitem,
     {
       widget = terminal_screen_get_widget (window->priv->active_term);
       
-      zvt_term_copy_clipboard (ZVT_TERM (widget));
+      terminal_widget_copy_clipboard (widget);
     }
 }
 
@@ -1547,8 +1598,8 @@ paste_callback (GtkWidget      *menuitem,
   if (window->priv->active_term)
     {
       widget = terminal_screen_get_widget (window->priv->active_term);
-      
-      zvt_term_paste_clipboard (ZVT_TERM (widget));
+
+      terminal_widget_paste_clipboard (widget);
     }  
 }
 
@@ -1649,6 +1700,15 @@ fullscreen_callback (GtkWidget      *menuitem,
                                   !terminal_window_get_fullscreen (window));
 }
 
+static void
+set_title_callback (GtkWidget      *menuitem,
+                    TerminalWindow *window)
+{
+  if (window->priv->active_term)
+    terminal_screen_edit_title (window->priv->active_term,
+                                GTK_WINDOW (window));
+}
+
 static void
 reset_callback (GtkWidget      *menuitem,
                 TerminalWindow *window)
@@ -1658,8 +1718,8 @@ reset_callback (GtkWidget      *menuitem,
   if (window->priv->active_term)
     {
       widget = terminal_screen_get_widget (window->priv->active_term);
-        
-      zvt_term_reset (ZVT_TERM (widget), FALSE);
+
+      terminal_widget_reset (widget, FALSE);
     }
 }
 
@@ -1672,8 +1732,8 @@ reset_and_clear_callback (GtkWidget      *menuitem,
   if (window->priv->active_term)
     {
       widget = terminal_screen_get_widget (window->priv->active_term);
-        
-      zvt_term_reset (ZVT_TERM (widget), TRUE);
+
+      terminal_widget_reset (widget, TRUE);
     }
 }
 
@@ -1721,6 +1781,9 @@ about_callback (GtkWidget      *menuitem,
                 TerminalWindow *window)
 {
   static GtkWidget *about = NULL;
+  GdkPixbuf *pixbuf = NULL;
+  gchar *file;
+
   const char *authors[] = {
     "Havoc Pennington <hp@redhat.com>",
     NULL
@@ -1728,21 +1791,25 @@ about_callback (GtkWidget      *menuitem,
   const char *documenters [] = {
     NULL
   };
-  const char *translator_credits = "";
+  const char *translator_credits = _("translator_credits");
 
   if (about)
     {
       gtk_window_present (GTK_WINDOW (about));
       return;
     }
+				     
+  file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-terminal.png", TRUE, NULL);
+  pixbuf = gdk_pixbuf_new_from_file (file, NULL);
+  g_free(file);
 
   about = gnome_about_new (PACKAGE, VERSION,
                            _("Copyright 2002 Havoc Pennington"),
                            _("GNOME Terminal"),
                            (const char **)authors,
                            (const char **)documenters,
-                           (const char *)translator_credits,
-                           NULL);
+                           strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+                           pixbuf);
   
   g_object_add_weak_pointer (G_OBJECT (about),
                              (void**) &about);
